#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var ts = require("typescript");
var yargs = require("yargs");
var bundle_generator_1 = require("../bundle-generator");
var check_diagnostics_errors_1 = require("../check-diagnostics-errors");
var get_compiler_options_1 = require("../get-compiler-options");
var logger_1 = require("../logger");
// tslint:disable-next-line:no-any
function toStringsArray(data) {
    if (!Array.isArray(data)) {
        throw new Error(data + " is not a array");
    }
    return data.map(String);
}
var args = yargs
    .usage('Usage: $0 [options] <file>')
    .demandCommand(1, 1)
    .option('out-file', {
    alias: 'o',
    type: 'string',
    description: 'File name of generated d.ts',
})
    .option('verbose', {
    type: 'boolean',
    default: false,
    description: 'Enable verbose logging',
})
    .option('no-check', {
    type: 'boolean',
    default: false,
    description: 'Skip validation of generated d.ts file',
})
    .option('fail-on-class', {
    type: 'boolean',
    default: false,
    description: 'Fail if generated dts contains class declaration',
})
    .option('external-inlines', {
    type: 'array',
    description: 'Array of package names from node_modules to inline typings from.\n' +
        'Used types will be inlined into the output file',
    coerce: toStringsArray,
})
    .option('external-imports', {
    type: 'array',
    description: 'Array of package names from node_modules to import typings from.\n' +
        'Used types will be imported using "import { First, Second } from \'library-name\';".\n' +
        'By default all libraries will be imported (except inlined libraries and libraries from @types)',
    coerce: toStringsArray,
})
    .option('external-types', {
    type: 'array',
    description: 'Array of package names from @types to import typings from via the triple-slash reference directive.\n' +
        'By default all packages are allowed and will be used according to their usages',
    coerce: toStringsArray,
})
    .option('umd-module-name', {
    type: 'string',
    description: 'Name of the UMD module. If specified then `export as namespace ModuleName;` will be emitted',
})
    .option('project', {
    type: 'string',
    description: 'Path to the tsconfig.json file that will be used for the compilation',
})
    .option('sort', {
    type: 'boolean',
    default: false,
    description: 'Sort output nodes',
})
    .option('inline-declare-global', {
    type: 'boolean',
    default: false,
    description: 'Enables inlining of `declare global` statements contained in files which should be inlined (all local files and packages from `--external-inlines`)',
})
    .option('disable-symlinks-following', {
    type: 'boolean',
    default: false,
    description: '(EXPERIMENTAL) Disables resolving of symlinks to the original path. See https://github.com/timocov/dts-bundle-generator/issues/39 for more information',
})
    .config('config', 'File path to the generator config file')
    .version()
    .strict()
    .example('$0 path/to/your/entry-file.ts', '')
    .example('$0 --external-types jquery react -- entry-file.ts', '')
    .wrap(Math.min(100, yargs.terminalWidth()))
    .argv;
if (args.verbose) {
    logger_1.enableVerbose();
}
try {
    var inputFilePath = args._[0];
    var generatedDts = bundle_generator_1.generateDtsBundle(inputFilePath, {
        failOnClass: args['fail-on-class'],
        inlinedLibraries: args['external-inlines'],
        importedLibraries: args['external-imports'],
        allowedTypesLibraries: args['external-types'],
        umdModuleName: args['umd-module-name'],
        preferredConfigPath: args.project,
        sortNodes: args.sort,
        inlineDeclareGlobals: args['inline-declare-global'],
        followSymlinks: !args['disable-symlinks-following'],
    });
    var outFile = args['out-file'];
    if (outFile === undefined) {
        var inputFileName = path.parse(inputFilePath).name;
        outFile = path.join(inputFilePath, '..', inputFileName + '.d.ts');
    }
    logger_1.normalLog("Writing generated file to " + outFile + "...");
    ts.sys.writeFile(outFile, generatedDts);
    if (args['no-check']) {
        logger_1.normalLog('File checking is skipped due to "no-check" flag');
        process.exit(0);
    }
    logger_1.normalLog('Checking the generated file...');
    var program = ts.createProgram([outFile], get_compiler_options_1.getCompilerOptions(inputFilePath, args.project));
    check_diagnostics_errors_1.checkProgramDiagnosticsErrors(program);
    logger_1.normalLog('Done.');
}
catch (ex) {
    console.error("Error: " + ex.message);
    process.exit(1);
}
