"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var typescript_helpers_1 = require("./typescript-helpers");
function generateOutput(params, options) {
    if (options === void 0) { options = {}; }
    var resultOutput = '';
    if (params.typesReferences.size !== 0) {
        var header = generateReferenceTypesDirective(Array.from(params.typesReferences));
        resultOutput += header + "\n\n";
    }
    if (params.imports.size !== 0) {
        // we need to have sorted imports of libraries to have more "stable" output
        var sortedEntries = Array.from(params.imports.entries()).sort(function (firstEntry, secondEntry) {
            return firstEntry[0].localeCompare(secondEntry[0]);
        });
        var importsArray = sortedEntries.map(function (entry) {
            var libraryName = entry[0], libraryImports = entry[1];
            return generateImport(libraryName, Array.from(libraryImports));
        });
        resultOutput += importsArray.join('\n') + "\n\n";
    }
    var statements = params.statements.map(function (statement) {
        return getStatementText(statement, params.shouldStatementHasExportKeyword(statement), params.needStripDefaultKeywordForStatement(statement));
    });
    if (options.sortStatements) {
        statements.sort();
    }
    resultOutput += statements.join('\n');
    if (options.umdModuleName !== undefined) {
        resultOutput += "\n\nexport as namespace " + options.umdModuleName + ";\n";
    }
    return resultOutput;
}
exports.generateOutput = generateOutput;
function getStatementText(statement, shouldStatementHasExportKeyword, needStripDefaultKeyword) {
    var hasStatementExportKeyword = ts.isExportAssignment(statement) || typescript_helpers_1.hasNodeModifier(statement, ts.SyntaxKind.ExportKeyword);
    var nodeText = getTextAccordingExport(statement.getText(), hasStatementExportKeyword, shouldStatementHasExportKeyword);
    // strip the `default` keyword from node
    if (typescript_helpers_1.hasNodeModifier(statement, ts.SyntaxKind.DefaultKeyword) && needStripDefaultKeyword) {
        // we need just to remove `default` from any node except class node
        // for classes we need to replace `default` with `declare` instead
        nodeText = nodeText.replace(/\bdefault\s/, ts.isClassDeclaration(statement) ? 'declare ' : '');
    }
    // for some reason TypeScript allows to do not write `declare` keyword for ClassDeclaration
    // if it already has `export` keyword - so we need to add it
    if (ts.isClassDeclaration(statement) && /^class\b/.test(nodeText)) {
        nodeText = "declare " + nodeText;
    }
    // add jsdoc for exported nodes only
    if (shouldStatementHasExportKeyword) {
        var start = statement.getStart();
        var jsDocStart = statement.getStart(undefined, true);
        var nodeJSDoc = statement.getSourceFile().getFullText().substring(jsDocStart, start).trim();
        if (nodeJSDoc.length !== 0) {
            nodeText = nodeJSDoc + "\n" + nodeText;
        }
    }
    return spacesToTabs(nodeText);
}
function generateImport(libraryName, imports) {
    // sort to make output more "stable"
    return "import { " + imports.sort().join(', ') + " } from '" + libraryName + "';";
}
function generateReferenceTypesDirective(libraries) {
    return libraries.sort().map(function (library) {
        return "/// <reference types=\"" + library + "\" />";
    }).join('\n');
}
function getTextAccordingExport(nodeText, isNodeExported, shouldNodeBeExported) {
    if (shouldNodeBeExported && !isNodeExported) {
        return 'export ' + nodeText;
    }
    else if (isNodeExported && !shouldNodeBeExported) {
        return nodeText.slice('export '.length);
    }
    return nodeText;
}
function spacesToTabs(text) {
    return text.replace(/^(    )+/gm, function (substring) {
        return '\t'.repeat(substring.length / 4);
    });
}
